from cherrypy.lib import cptools
from cherrypy import _cputil
import cherrypy

import sys
import traceback
import types

import forms, api, config

def _expose(func, alias):
    """Expose the function, optionally providing an alias or set of aliases."""
    
    def expose_(func):
        func.exposed = True
        if alias is not None:
            if isinstance(alias, basestring):
                parentDict[alias] = func
            else:
                for a in alias:
                    parentDict[a] = func
        return func

    parentDict = sys._getframe(4).f_locals
    return expose_(func)

def expose(alias=None):
    """
    Let's have both @expose and if needed to have aliases, @expose(aliases)
    """
    def decorator(m):
        def _m(*args, **kwargs):
            forms.form_defaults.update(api.request.args)
            try:
                return m(*args, **kwargs)
            finally:
                forms.form_errors.clear()
                forms.form_defaults.clear()
                if config.config_get("subway.no_cache", False):
                    api.response.no_cache()
                cherrypy.threadData.form_submitted = False
        _m.__doc__ = m.__doc__
        return _expose(_m, alias)

    if callable(alias):
        # @expose
        func = alias
        alias = None
        return decorator(func)
    else:
        # @expose(alias)
        return lambda f: cptools.decorate(f, decorator)

class Controller(cptools.PositionalParametersAware):
    """
    Base Controller class.
    """
    
    def __init__(self, module):
        self._module = module

    @expose
    def default(self, *args, **kwargs):
        return cptools.PositionalParametersAware.default(self, *args, **kwargs)
    
    def __getattr__(self, k):
        return getattr(self._module, k)

    def _cpOnError(self):
        body = cherrypy.response.body # Save old body (changed by ErrorFilter)
        _cputil._cpOnError() # Standard CP error
        if body:
            cherrypy.response.body = body # Put ErrorFilter body after HTTPError.set_response() changed it
