import imp
import os
import sys
import inspect

import namespaces

import subway

from sqlobject.declarative import DeclarativeMeta

def magic_import(code, filename, namespace, mod=None):
    if not mod:
        modname = os.path.splitext(os.path.basename(filename))[0]
        mod = imp.new_module(modname)
        mod.__file__ = filename
        sys.modules[modname] = mod
    for (k,v) in namespace.items():
        setattr(mod, k, v)
    code = compile(code, filename, "exec")
    exec code in mod.__dict__
    return mod

def import_with_superglobals(filename):
    return magic_import(open(filename, "r").read(), filename, namespaces.get_superglobal_namespace())

def import_controller(filename, view_dir, base_view_dir, model):
    viewobj = subway.ViewManager(view_dir, base_view_dir)
    mod = magic_import(open(filename, "r").read(),
                        filename,
                        namespaces.get_controller_namespace(
                            viewobj,
                            model))
    return subway.Controller(mod)

def import_models(filenames):
    model = imp.new_module("model")
    sys.modules["model"] = model
    for filename in filenames:
        magic_import(open(filename, "r").read(),
                     filename,
                     namespaces.get_model_namespace(),
                     model)
        # for the benefit of autoreload...
        sys.modules["model_" + filename] = imp.new_module("model_" + filename)
        sys.modules["model_" + filename].__file__ = filename
    models = {}
    for objname in dir(model):
        obj = getattr(model, objname)
        if getattr(obj, "__module__", None) == 'model' and isinstance(obj, DeclarativeMeta):
            models[objname] = obj
    return models

def import_daemon(filename, model):
    mod = magic_import(open(filename, "r").read(),
                        filename,
                        namespaces.get_daemon_namespace(model))
    assert callable(getattr(mod, "main", None)), "Every daemon must have a callable main() function"
    return mod.main
    
