# view.py - easily accessible precompiled cheetah templates
import os
import imp
import sys
import sha

namespaces = None # lazy import

from Cheetah import Compiler, NameMapper, Template

import cherrypy

import forms, api

from formencode import htmlfill, htmlfill_schemabuilder

import re

RE_EXTENDS = re.compile(r"#extends\s+(.+)")
VIEW_HEADER = """import os
import subway
view = subway.ViewManager(os.path.split(os.path.dirname(os.path.abspath(__file__)))[0], %r)
"""

class ViewNotFound(LookupError):
    pass

class ViewManager(object):
    """
    """
    def __init__(self, path, absolute_root=None, default_namespace=None):
        self.path = path
        if not absolute_root:
            absolute_root = self.path
        self.absolute_root = absolute_root
        global namespaces
        if namespaces is None:
            from framework import namespaces
        if default_namespace:
            self.default_namespace = default_namespace
        else:
            self.default_namespace = namespaces.get_view_namespace()
        self.form_cache = {}
    def render(self, file_or_func, first_item=None, *search_list, **kwargs):
        sl = []
        if first_item:
            sl.append(first_item)
        if kwargs:
            sl.append(kwargs)
        if search_list:
            sl.extend(search_list)
        f = sys._getframe(1)
        sl.append(f.f_locals)
        sl.append(f.f_globals)
        
        sl.append(self.default_namespace)
        t = self.get(file_or_func)(searchList = sl)
        content = t.respond()
        if "_resume" in forms.form_defaults:
            del forms.form_defaults["_resume"] # we don't want to carry over the resume id
        errors = forms.form_errors
        if not errors:
            errors = None
        listener = htmlfill_schemabuilder.SchemaBuilder()
        p = htmlfill.FillingParser(forms.form_defaults, listener=listener, errors=errors)
        p.feed(content)
        if not callable(file_or_func):
            file_or_func = os.path.join(self.path, file_or_func)
        self.form_cache[file_or_func] = listener.schema()
        p.close()
        fe_content = p.text()
        if fe_content:
            content = fe_content
        return content
    def get(self, file_or_func):
        """
        Returns subclass of Template
        file - source filename to compile, OR a function or method with a docstring

        this method cheetah compiles the source file if it is newer than the compiled version    
        """
        if callable(file_or_func):
            content = getattr(file_or_func, "__doc__", "")
            file_or_func = "tmpl_%s.txt" % sha.new(content).hexdigest()
            file = os.path.join(self.path, file_or_func)
            if not os.path.isfile(file):
                open(file, "w").write(content)
        else:
            if len(file_or_func) > 0 and file_or_func[0] in ('/', '\\'):
                file = os.path.join(self.absolute_root, file_or_func[1:])
            else:
                file = os.path.join(self.path, file_or_func)
        tmpldir, filename = os.path.split(file)
        precompile_dir = os.path.join(tmpldir, ".compiled")
        pathnoext, ext = os.path.splitext(file)
        tname = os.path.basename(pathnoext) + "_tmpl"
        compiledfile = os.path.join(precompile_dir, tname + ".py")
        if not os.path.exists(precompile_dir):
            os.mkdir(precompile_dir)
        if not os.path.exists(compiledfile) or os.stat(file).st_mtime > os.stat(compiledfile).st_mtime:
            # compiled file either does not exist, or is older than the source.
            # recompile the template and clear any cached data
            if file_or_func in self.form_cache:
                del self.form_cache[file_or_func]
            try:
                raw_source = open(file, "r").read()
            except:
                raise ViewNotFound, "file %r is not accessible" % file
            ## this next chunk figures out all of the classes
            ## this template #extends, and puts it into extendlist
            extend_expr = [] # needs to be a container because of lame python warts
            def _sub(match):
                if match.group(1).startswith("view"):
                    # if extending a view-prefixed class, munge the name
                    # because view-prefixed base classes need to use magic
                    #e = match.group(1)[match.group(1).find(".")+1:].replace(".", "_dot_")
                    if extend_expr:
                        raise "Your template must only extend one "
                    extend_expr.append(match.group(1))
                    return "#extends BaseClass"
                else:
                    return match.group(0)
            source = RE_EXTENDS.sub(_sub, raw_source)
            # now, compile the file
            code = (VIEW_HEADER % self.absolute_root) + str(Compiler.Compiler(source=source, moduleName=tname, mainClassName=tname)) + "\n%s.raw_source = %r" % (tname, raw_source)
            if extend_expr:
                code = code.replace("from BaseClass import BaseClass", "BaseClass = " + extend_expr[0])
            
            ## replace cheetah-generated view imports to subway view lookups
            #for extend in extendlist:
            #    code = code.replace("from %s import %s" % (extend, extend), "%s = view.%s.template_class" % (extend, extend.replace("_dot_",".")))
            # write out the compiled file including boilerplate header
            open(compiledfile, 'w').write(code)
        # import the template
        fp, pathname, description = imp.find_module(tname, [precompile_dir])
        try:
            a = imp.load_module(tname, fp, pathname, description)
        finally: # Since we may exit via an exception, close fp explicitly.
            if fp:
                fp.close()
        return getattr(a, tname) # return the class extracted from the module
    def form(self, file_or_func):
        if file_or_func not in self.form_cache:
            tmpl = self.get(file_or_func)
            listener = htmlfill_schemabuilder.SchemaBuilder()
            p = htmlfill.FillingParser({}, listener=listener)
            p.feed(tmpl.raw_source)
            p.close()
            self.form_cache[file_or_func] = listener.schema()
        return self.form_cache[file_or_func]
    def __repr__(self):
        return "<ViewManager for %r>" % self.path
    __str__ = __repr__
